#version 330 core


in vec2 UV;
in vec3 vertexPosition_worldspace;
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace;
in vec3 lightDirection_cameraspace;

// Ouput data
out vec4 color;

uniform sampler2D textureBitmap;
uniform vec3 LightPosition_worldspace;

void main()
{
	vec3 LightColor = vec3(1,1,1);
	float LightPower = 50.0f;
	
	// Material properties
	vec3 MaterialDiffuseColor = texture( textureBitmap, UV ).rgb;
	vec3 MaterialAmbientColor = vec3(0.5,0.5,0.5) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.3,0.3,0.3);

	// Distance to the light
	float distance = length(LightPosition_worldspace - vertexPosition_worldspace );
	vec3 n = normalize( normal_cameraspace );
	// dir from fragment to light
	vec3 l = normalize( lightDirection_cameraspace );
	
	float cosTheta = clamp( dot( n,l ), 0,1 );

	vec3 E = normalize(eyeDirection_cameraspace);

	vec3 R = reflect(-l,n);
	
	float cosAlpha = clamp( dot( E,R ), 0,1 );

	color = vec4(MaterialAmbientColor + MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance) + MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5) / (distance*distance),texture( textureBitmap, UV ).w);

	//color = vec4(MaterialAmbientColor,1);

}