#version 330 core


in vec2 UV;
in vec3 vertexPosition_worldspace;
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace;
in vec3 lightDirection_cameraspace;
in vec4 ShadowCoord;

in vec3 F_lightDirection_cameraspace;


// Ouput data
out vec4 color;

uniform sampler2D textureBitmap;
uniform sampler2DShadow shadowMap;
uniform vec3 LightPosition_worldspace;

uniform vec2 windowSize;

uniform vec3 pPos;
uniform vec3 pDir;




void main()
{
	
	/////////// Basic
	vec3 LightColor = vec3(1,1,1);
	// Material properties
	vec3 MaterialDiffuseColor = texture( textureBitmap, UV ).rgb;
	vec3 MaterialAmbientColor = vec3(0.15,0.15,0.15) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.3,0.3,0.3);

	//////////

	////////// MoonLight
	float LightPower = 50.0f;
	float distance = length(LightPosition_worldspace - vertexPosition_worldspace );
	vec3 n = normalize( normal_cameraspace );
	vec3 l = normalize( lightDirection_cameraspace );
	float cosTheta = clamp( dot( n,l ), 0,1 );
	vec3 E = normalize(eyeDirection_cameraspace);
	vec3 R = reflect(-l,n);	
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	//////////
	


	
	/////////// Flashlight
	float FLightPower = 10.0f;
	vec3 lightDir = normalize((pPos - vec3(0,0.5,0) + pDir) - vertexPosition_worldspace);
	float lightToSurfaceAngle = degrees(acos(dot(-lightDir, pDir)));
	float attenuation = 0.8;
	if(lightToSurfaceAngle > 90.0){
		attenuation = 0.0;
	}	
	float Fdistance = length((pPos - vec3(0,0.5,0) + pDir) - vertexPosition_worldspace );
	vec3 Fl = normalize( F_lightDirection_cameraspace );
	float FcosTheta = clamp( dot( n,Fl ), 0,1 );
	vec3 FR = reflect(-Fl,n);
	float FcosAlpha = clamp( dot( E,FR ), 0,1 );
	////////

	//
	//	FLASHLIGHT Circle
	//
	vec3 sm_ndc = ShadowCoord.xyz / ShadowCoord.w;

	//////////// calc pixelsize and make flashlight round first
	vec2 p = sm_ndc.xy * windowSize;                  //vec2(800,600);
    //vec2 m = vec2(400.0,300.0) / vec2(800.0,600.0);
	vec2 m = windowSize / 2.0;  // vec2(400.0,300.0);
    vec2 d = p - m;	
	float r = sqrt(dot(d, d)); // distance of pixel from mouse
	float sizeFlashlight = windowSize.y / 3;		// 190.0;
 
	float flashlightPower = 15.0; 
	////////////

	/// calc std lighting
	vec3 adiff =  MaterialDiffuseColor * LightColor * LightPower * ((cosTheta / (distance*distance)));
	vec3 aspec =  MaterialSpecularColor * LightColor * LightPower * (pow(cosAlpha,5) / (distance*distance));


	// test radius
    if (r > sizeFlashlight) 
    {
		
		color = vec4(MaterialAmbientColor +   adiff    +  aspec,texture( textureBitmap, UV ).w);
        
    } else {
		
		
		float depth_p = ShadowCoord.z / ShadowCoord.w;
		
		float shadowMult = 0;

		//const float kTransparency = 0.3;
		float mapScale = 1.0 / 512.0;
		
		//float depth_c = texture(shadowMap, sm_ndc);
		//shadowMult += (depth_c < depth_p) ? 1.0 : (sizeFlashlight/r * 1.1 < 5.0 ? sizeFlashlight/r * 1.1 : 5.0) ;

		float depth_c = 0;
		float count = 0.0;
		for(int i = -2; i < 3; i++){
			for(int j = -2; j < 3; j++){
				depth_c = texture(shadowMap, sm_ndc + vec3( i * mapScale, j * mapScale, 0));
				shadowMult += (depth_c < depth_p) ? 1.0 : (sizeFlashlight/r * 1.1 < flashlightPower ? sizeFlashlight/r * 1.1 : flashlightPower) ;
				count++;
			}
		}
		
		// add diff and spec lighting to flashlight
		vec3 bdiff =  (sizeFlashlight - r)/100.0f * attenuation * MaterialDiffuseColor * LightColor * FLightPower * ((FcosTheta / (Fdistance*Fdistance)));
		vec3 bspec =  (sizeFlashlight - r)/100.0f * attenuation * MaterialSpecularColor * LightColor * FLightPower * (pow(FcosAlpha,5) / (Fdistance*Fdistance));

		shadowMult = shadowMult / count;
		
		color = vec4(shadowMult * MaterialAmbientColor +shadowMult *  (adiff+bdiff) + shadowMult *  (aspec + bspec),texture( textureBitmap, UV ).w);
	}
}